﻿using System;
using System.Windows;
using Microsoft.Win32;
using System.IO;
using System.Windows.Media;
using System.Windows.Documents;
using System.Windows.Controls;
using System.Windows.Markup;

namespace CajasDeDialogo
{
  /// <summary>
  /// Lógica de interacción para MainWindow.xaml
  /// </summary>
  public partial class MainWindow : Window
  {
    enum ResultadoDlg {Aceptar, Cancelar, Error}; // Aceptar, Cancelar o Error

    public MainWindow()
    {
      InitializeComponent();

      // Se admiten tres intentos para introducir la palabra de paso
      int nCuenta = 0;
      ResultadoDlg vr; // Aceptar, Cancelar o Error
      do
      {
        nCuenta += 1;
        vr = contraseña();
      }
      while (vr == ResultadoDlg.Error && nCuenta < 3);
      if (vr == ResultadoDlg.Cancelar || nCuenta == 3) Close();
    }

    private ResultadoDlg contraseña()
    {
      string sPalabraDePaso;
      // Crear la caja de diálogo de entrada
      DlgPalabraDePaso Dlg = new DlgPalabraDePaso();

      // Mostrar el diálogo
      if (Dlg.ShowDialog() == true)
        // Obtener los datos
        sPalabraDePaso = Dlg.ctPalabraDePaso.Password;
      else
        return ResultadoDlg.Cancelar;

      // Verificar si la palabra de paso es correcta
      string pp = Properties.Settings.Default.PalabraDePaso;
      if (sPalabraDePaso.CompareTo(pp) != 0)
      {
        MessageBox.Show("La palabra de paso no es correcta", "Error",
                        MessageBoxButton.OK, MessageBoxImage.Error);
        return ResultadoDlg.Error;
      }
      return ResultadoDlg.Aceptar;
    }

    private void ArchivoSalir_Click(object sender, RoutedEventArgs e)
    {
      Close();
    }

    private void AyudaAcercaDe_Click(object sender, RoutedEventArgs e)
    {
      AcercaDe dlg = new AcercaDe();
      dlg.ShowDialog();
    }

    private void DialogoCasillaV_Click(object sender, RoutedEventArgs e)
    {
      DlgCasillaVerificacion dlg = new DlgCasillaVerificacion();
      dlg.ShowDialog();
    }

    private void DialogoBtOpcion_Click(object sender, RoutedEventArgs e)
    {
      DlgBasesNum dlg = new DlgBasesNum();
      dlg.ShowDialog();
    }

    private void DialogoListaSimple_Click(object sender, RoutedEventArgs e)
    {
      DlgListaSimple dlg = new DlgListaSimple();
      dlg.ShowDialog();
    }

    private void DialogoListaDesplegable_Click(object sender, RoutedEventArgs e)
    {
      DlgListaDesplegable dlg = new DlgListaDesplegable();
      dlg.ShowDialog();
    }

    private void DialogoRangos_Click(object sender, RoutedEventArgs e)
    {
      DlgRangos dlg = new DlgRangos();
      dlg.ShowDialog();
    }

    private void Calendario_Click(object sender, RoutedEventArgs e)
    {
      DlgCalendario dlg = new DlgCalendario();
      dlg.ShowDialog();
    }

    private void DialogoListView_Click(object sender, RoutedEventArgs e)
    {
      DlgListView dlg = new DlgListView();
      dlg.ShowDialog();
    }

    private void DialogoTreeView_Click(object sender, RoutedEventArgs e)
    {
      DlgTreeView dlg = new DlgTreeView();
      dlg.ShowDialog();
    }

    private void DialogoDataGrid_Click(object sender, RoutedEventArgs e)
    {
      DlgDataGrid dlg = new DlgDataGrid();
      dlg.ShowDialog();
    }

    private void DialogoColor_Click(object sender, RoutedEventArgs e)
    {
      System.Windows.Forms.ColorDialog DlgColor =
        new System.Windows.Forms.ColorDialog();

      // Seleccionar inicialmente el color actual del fondo
      SolidColorBrush cta = richTextBox.Foreground as SolidColorBrush;
      DlgColor.Color = System.Drawing.Color.FromArgb(
        cta.Color.A, cta.Color.R, cta.Color.G, cta.Color.B);
      // Actualizar el color del fondo de TextBox1
      if (DlgColor.ShowDialog() == System.Windows.Forms.DialogResult.OK)
      {
        var wpfColor = System.Windows.Media.Color.FromArgb(
          DlgColor.Color.A,
          DlgColor.Color.R, DlgColor.Color.G, DlgColor.Color.B);
        var ctn = new SolidColorBrush(wpfColor);
        richTextBox.Foreground = ctn;
      }
    }

    private void ArchivoAbrir_Click(object sender, RoutedEventArgs e)
    {
      OpenFileDialog DlgAbrir = new OpenFileDialog();

      DlgAbrir.Filter = "ficheros xaml (*.xaml)|*.xaml" +
        "|ficheros rtf (*.rtf)|*.rtf|Todos (*.*)|*.*";
      DlgAbrir.FilterIndex = 2;
      // Mostrar el diálogo Abrir
      if (DlgAbrir.ShowDialog() == true)
      {
        FlowDocument flowDocument = new FlowDocument();
        TextRange textRange = new TextRange(flowDocument.ContentStart, flowDocument.ContentEnd);
        FileStream fStream = new FileStream(DlgAbrir.FileName, FileMode.Open);
        if (DlgAbrir.FileName.ToUpper().EndsWith(".XAML"))
          textRange.Load(fStream, DataFormats.Xaml);
        else
          textRange.Load(fStream, DataFormats.Rtf);
        fStream.Close();
        richTextBox.Document = flowDocument;
      }
    }

    private void ArchivoGuardar_Click(object sender, RoutedEventArgs e)
    {
      SaveFileDialog DlgGuardar = new SaveFileDialog();

      DlgGuardar.Filter = "ficheros xaml (*.xaml)|*.xaml" +
        "|ficheros rtf (*.rtf)|*.rtf|Todos (*.*)|*.*";
      DlgGuardar.FilterIndex = 2;

      if (DlgGuardar.ShowDialog() == true)
      {
        FileStream fStream;
        if (richTextBox.Document != null)
        {
          TextRange selección = new TextRange(richTextBox.Document.ContentStart, richTextBox.Document.ContentEnd);
          fStream = new FileStream(DlgGuardar.FileName, FileMode.Create);
          if (DlgGuardar.FileName.ToUpper().EndsWith(".XAML"))
            selección.Save(fStream, DataFormats.Xaml);
          else
            selección.Save(fStream, DataFormats.Rtf);
          fStream.Close();
        }
      }
    }

    private void ArchivoImprimir_Click(object sender, RoutedEventArgs e)
    {
      string descripcionDoc = "Documento";
      PrintDialog DlgImprimir = new PrintDialog();
      if (DlgImprimir.ShowDialog() == true)
      {
        //DlgImprimir.PrintVisual(richTextBox, descripcionDoc);

        //// Imprimir el documento paginado existente
        //IDocumentPaginatorSource docFuente = richTextBox.Document;
        //DlgImprimir.PrintDocument(docFuente.DocumentPaginator, descripcionDoc);

        FlowDocument docFuente = richTextBox.Document;

        // Guardar la configuración actual                                
        double pageWidth = docFuente.PageWidth;
        double pageHeight = docFuente.PageHeight;
        double columnGap = docFuente.ColumnGap;
        double columnWidth = docFuente.ColumnWidth;
        Thickness pagePadding = docFuente.PagePadding;

        // Hacer coincidir la página FlowDocument con la página impresa
        docFuente.PageHeight = DlgImprimir.PrintableAreaHeight;
        docFuente.PageWidth = DlgImprimir.PrintableAreaWidth;
        docFuente.PagePadding = new Thickness(50);

        // Utilizar dos columnas
        docFuente.ColumnGap = 24;
        docFuente.ColumnWidth = (docFuente.PageWidth - docFuente.ColumnGap -
          docFuente.PagePadding.Left - docFuente.PagePadding.Right) / 2;

        IDocumentPaginatorSource docFuentePag = docFuente;
        DlgImprimir.PrintDocument(docFuentePag.DocumentPaginator, descripcionDoc);

        // Restablecer la configuración inicial
        docFuente.PageWidth = pageWidth;
        docFuente.PageHeight = pageHeight;
        docFuente.ColumnGap = columnGap;
        docFuente.ColumnWidth = columnWidth;
        docFuente.PagePadding = pagePadding;
      }
    }

    private void DialogoDocDinamico_Click(object sender, RoutedEventArgs e)
    {
      DlgDocDinamico dlg = new DlgDocDinamico();
      dlg.ShowDialog();
    }
  }
}
