﻿using System.Text;
using System.Windows;
using System.Windows.Data;
using System.ComponentModel;
using BaseDeDatos;
using System.Collections.ObjectModel;

namespace EnlaceDeDatosObjetos
{
  /// <summary>
  /// Lógica de interacción para DlgOperacionesObjs.xaml
  /// </summary>
  public partial class DlgOperacionesObjs : Window
  {
    public DlgOperacionesObjs()
    {
      InitializeComponent();
    }

    private ICollectionView ObtenerVista()
    {
      return CollectionViewSource.GetDefaultView(listaAlumnos.ItemsSource);
    }

    private void btMostrarOb_Click(object sender, RoutedEventArgs e)
    {
      MostrarAsigs("Obligatorias");
    }

    private void btMostrarOp_Click(object sender, RoutedEventArgs e)
    {
      MostrarAsigs("Optativas");
    }

    private void MostrarAsigs(string tipo)
    {
      ICollectionView vista = ObtenerVista();
      if (vista.IsEmpty) return;
      // Cada "alumno" es un objeto de la clase Alumno
      Alumno alumnoActual = (Alumno)vista.CurrentItem;
      ObservableCollection<Asignatura> list = null;
      if (tipo == "Obligatorias")
        list = alumnoActual.ListaAsigsObs;
      else if (tipo == "Optativas")
        list = alumnoActual.ListaAsigsOps;
      else
        return;
      StringBuilder s = new StringBuilder();
      foreach (Asignatura asig in list)
        s.Append(asig.NomAsignatura + "\n");
      MessageBox.Show(s.ToString());
    }

    private void btSiguiente_Click(object sender, RoutedEventArgs e)
    {
      ICollectionView vista = ObtenerVista();
      vista.MoveCurrentToNext();
      if (vista.IsCurrentAfterLast) vista.MoveCurrentToLast();
      listaAlumnos.ScrollIntoView(vista.CurrentItem);
    }

    private void btAnterior_Click(object sender, RoutedEventArgs e)
    {
      ICollectionView vista = ObtenerVista();
      vista.MoveCurrentToPrevious();
      if (vista.IsCurrentBeforeFirst) vista.MoveCurrentToFirst();
      listaAlumnos.ScrollIntoView(vista.CurrentItem);
    }

    private void cvOrdenar_Click(object sender, RoutedEventArgs e)
    {
      ICollectionView vista = ObtenerVista();
      if (vista.CanSort && cvOrdenar.IsChecked == true)
      {
        if (vista.SortDescriptions.Count == 0)
        {
          vista.SortDescriptions.Add(
            new SortDescription("BecaAlumno", ListSortDirection.Descending));
          vista.SortDescriptions.Add(
            new SortDescription("NombAlumno", ListSortDirection.Ascending));
        }
      }
      else
        vista.SortDescriptions.Clear();
    }

    private void cvFiltrar_Click(object sender, RoutedEventArgs e)
    {
      ICollectionView vista = ObtenerVista();
      if (vista.CanFilter && cvFiltrar.IsChecked == true)
      {
        if (vista.Filter == null)
          vista.Filter = delegate(object elemento)
          {
            return ((Alumno)elemento).BecaAlumno == true;
          };
      }
      else
        vista.Filter = null;
    }

    private void cvAgrupar_Click(object sender, RoutedEventArgs e)
    {
      ICollectionView vista = ObtenerVista();
      if (vista.CanGroup && cvAgrupar.IsChecked == true)
      {
        if (vista.GroupDescriptions.Count == 0)
        {
          // Agrupar por BecaAlumno
          vista.GroupDescriptions.Add(new PropertyGroupDescription("BecaAlumno"));
        }
      }
      else
        vista.GroupDescriptions.Clear();
    }
  }
}
