﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Xml;

namespace EnlaceDeDatosXML
{
  /// <summary>
  /// Lógica de interacción para DlgOperacionesXml.xaml
  /// </summary>
  public partial class DlgOperacionesXml : Window
  {
    public DlgOperacionesXml()
    {
      InitializeComponent();
    }

    private ICollectionView ObtenerVista()
    {
      DataSourceProvider proveedor = (DataSourceProvider)this.FindResource("xmlAlumnos");
      return CollectionViewSource.GetDefaultView(proveedor.Data);
    }

    private void btMostrarOb_Click(object sender, RoutedEventArgs e)
    {
      MostrarAsigs("Obligatorias");
    }

    private void btMostrarOp_Click(object sender, RoutedEventArgs e)
    {
      MostrarAsigs("Optativas");
    }

    private void MostrarAsigs(string tipo)
    {
      ICollectionView vista = ObtenerVista();
      if (vista.IsEmpty) return;
      // Cada "alumno" es un XmlElement
      XmlElement alumnoActual = (XmlElement)vista.CurrentItem;
      XmlNodeList list = alumnoActual.GetElementsByTagName("asignaturas");
      foreach (XmlElement asigs in list)
        if (asigs.GetAttribute("tipo") == tipo)
        {
          list = asigs.GetElementsByTagName("asignatura");
          StringBuilder s = new StringBuilder();
          foreach (XmlElement asig in list)
            s.Append(asig.FirstChild.InnerText + "\n");
          MessageBox.Show(s.ToString());
        }
    }

    private void btSiguiente_Click(object sender, RoutedEventArgs e)
    {
      ICollectionView vista = ObtenerVista();
      vista.MoveCurrentToNext();
      if (vista.IsCurrentAfterLast) vista.MoveCurrentToLast();
      listaAlumnos.ScrollIntoView(vista.CurrentItem);
    }

    private void btAnterior_Click(object sender, RoutedEventArgs e)
    {
      ICollectionView vista = ObtenerVista();
      vista.MoveCurrentToPrevious();
      if (vista.IsCurrentBeforeFirst) vista.MoveCurrentToFirst();
      listaAlumnos.ScrollIntoView(vista.CurrentItem);
    }

    private void cvOrdenar_Click(object sender, RoutedEventArgs e)
    {
      ICollectionView vista = ObtenerVista();
      if (vista.CanSort && cvOrdenar.IsChecked == true)
      {
        if (vista.SortDescriptions.Count == 0)
        {
          vista.SortDescriptions.Add(
            new SortDescription("beca", ListSortDirection.Descending));
          vista.SortDescriptions.Add(
            new SortDescription("nombre", ListSortDirection.Ascending));
        }
      }
      else
        vista.SortDescriptions.Clear();
    }

    private void cvFiltrar_Click(object sender, RoutedEventArgs e)
    {
      ICollectionView vista = ObtenerVista();
      if (vista.CanFilter && cvFiltrar.IsChecked == true)
      {
        if (vista.Filter == null)
          vista.Filter = delegate(object elemento)
          {
            return ((XmlElement)elemento).ChildNodes[4].InnerText == "True";
          };
      }
      else
        vista.Filter = null;
    }

    private void cvAgrupar_Click(object sender, RoutedEventArgs e)
    {
      ICollectionView vista = ObtenerVista();
      if (vista.CanGroup && cvAgrupar.IsChecked == true)
      {
        if (vista.GroupDescriptions.Count == 0)
        {
          // Agrupar por beca
          vista.GroupDescriptions.Add(new PropertyGroupDescription("beca"));
        }
      }
      else
        vista.GroupDescriptions.Clear();
    }
  }
}
