﻿using System.Windows;
using System.Data.SqlClient;
using System.Data;
using System.Collections.ObjectModel;
using System.Transactions;

namespace Transacciones
{
  /// <summary>
  /// Lógica de interacción para MainWindow.xaml
  /// </summary>
  public partial class MainWindow : Window
  {
    private SqlConnection ConexionConBD = null;
    private SqlCommand OrdenSql;
    private SqlDataReader Lector;
    private string strConexion = null;

    public MainWindow()
    {
      InitializeComponent();
    }

    private void Window_Loaded(object sender, RoutedEventArgs e)
    {
      // Crear la cadena de conexión
      strConexion = "Data Source=.\\sqlexpress;" +
        "Initial Catalog=bd_telefonos; Integrated Security=True";
    }

    private void btMostrarDatos_Click(object sender, RoutedEventArgs e)
    {
      try
      {
        using (ConexionConBD = new SqlConnection(strConexion))
        {
          // Procedimiento almacenado
          OrdenSql = new SqlCommand("stproObtenerTfnos", ConexionConBD);
          OrdenSql.CommandType = CommandType.StoredProcedure;
          OrdenSql.Parameters.AddWithValue("@prefijo", ctSql.Text);

          // Abrir la base de datos
          ConexionConBD.Open();
          // ExecuteReader hace la consulta y devuelve un SqlDataReader
          Lector = OrdenSql.ExecuteReader();
          // Colección de datos obtenida de la consulta
          ObservableCollection<CTelefono> lista = new ObservableCollection<CTelefono>();
          string nombre, tfno;
          // Llamar siempre a Read antes de acceder a los datos
          while (Lector.Read()) // siguiente registro
          {
            nombre = Lector["nombre"] as string;
            tfno = Lector["telefono"] as string;
            lista.Add(new CTelefono(nombre, tfno));
          }
          // Llamar siempre a Close una vez finalizada la lectura
          Lector.Close();
          // Contextos de datos
          lsTfnos.DataContext = lista;
          ctTfno.DataContext = lsTfnos;
        }
      }
      catch (System.Exception ex)
      {
        MessageBox.Show(ex.Message);
      }
    }

    private void btGuardarDatos_Click(object sender, RoutedEventArgs e)
    {
      try
      {
        string strTelefonoAntiguo = lsTfnos.SelectedValue as string;
        string strTelefonoNuevo = ctTfno.Text;
        if (strTelefonoAntiguo == strTelefonoNuevo) return;
        using (TransactionScope tr = new TransactionScope())
        {
          using (ConexionConBD = new SqlConnection(strConexion))
          {
            // Procedimiento almacenado
            OrdenSql = new SqlCommand("stproModificarTfno", ConexionConBD);
            OrdenSql.CommandType = CommandType.StoredProcedure;
            OrdenSql.Parameters.AddWithValue("@tfnoAntiguo", strTelefonoAntiguo);
            OrdenSql.Parameters.AddWithValue("@tfnoNuevo", strTelefonoNuevo);

            // Abrir la base de datos
            ConexionConBD.Open();
            // Ejecutar la sentencia SQL
            OrdenSql.ExecuteNonQuery();
          }
          tr.Complete();
        }
        // Actualizar los datos de la lista
        btMostrarDatos_Click(sender, e);
      }
      catch (System.Exception ex)
      {
        MessageBox.Show(ex.Message);
      }
    }
  }
}
