﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using BDNotasAlumnos;
using System.ComponentModel;
using System.Data.Objects;
using System.Data;

namespace ApNotasAlumnos
{
  /// <summary>
  /// Lógica de interacción para MainWindow.xaml
  /// </summary>
  public partial class MainWindow : Window
  {
    private object asigs;

    public MainWindow()
    {
      InitializeComponent();
    }

    private void dgAlumnos_SelectionChanged(object sender, SelectionChangedEventArgs e)
    {
      if (e.AddedItems.Count == 0) return;
      alumno alum = e.AddedItems[0] as alumno;
      if (alum == null) return;
      CProveedorAsignaturasNotas pAsigsNotas = new CProveedorAsignaturasNotas();
      asigs = pAsigsNotas.ObtenerAsignaturasNotas(alum.id_alumno);
      spAsignaturas.DataContext = asigs;
    }

    private void boTodas_Checked(object sender, RoutedEventArgs e)
    {
      ICollectionView vista = CollectionViewSource.GetDefaultView(asigs);
      if (vista == null) return;
      vista.Filter = null;
    }

    private void boSuspensas_Checked(object sender, RoutedEventArgs e)
    {
      ICollectionView vista = CollectionViewSource.GetDefaultView(asigs);
      if (vista == null) return;
      vista.Filter = delegate(object elemento)
      {
        dynamic elem = elemento;
        float nota = elem.nota;
        return elem.nota < 5;
      };
    }

    private void boAprobadas_Checked(object sender, RoutedEventArgs e)
    {
      ICollectionView vista = CollectionViewSource.GetDefaultView(asigs);
      if (vista == null) return;
      vista.Filter = delegate(object elemento)
      {
        dynamic elem = elemento;
        float nota = elem.nota;
        return elem.nota >= 5;
      };
    }

    private void ArchivoSalir_Click(object sender, RoutedEventArgs e)
    {
      Close();
    }

    private void CambiosBDModificarNota_Click(object sender, RoutedEventArgs e)
    {
      using (bd_notasAlumnosEntities contextoDeObjs =
             new bd_notasAlumnosEntities())
      {
        // Datos a obtener del diálogo 
        int idAlum;
        int idAsig;
        float notaAlum;

        // Mostrar el diálogo
        ModificarNota dlgModNota = new ModificarNota();
        if (dlgModNota.ShowDialog() == true)
        {
          // Obtener los datos
          idAlum = dlgModNota.IdAlumno;
          idAsig = dlgModNota.IdAsignatura;
          notaAlum = dlgModNota.Nota;
        }
        else
          return;

        // Consulta para obtener la fila a modificar
        var consulta =
            from al_as in contextoDeObjs.alums_asigs
            where al_as.id_alumno == idAlum &&
                  al_as.id_asignatura == idAsig
            select al_as;

        if (consulta.Count() == 0)
        {
          MessageBox.Show("La consulta no contiene elementos");
          return;
        }

        // Ejecutar la consulta y cambiar los valores
        // de las columnas implicadas
        foreach (alum_asig al_as in consulta)
          al_as.nota = notaAlum;

        // Enviar los cambios a la base de datos
        try
        {
          contextoDeObjs.SaveChanges();
          MessageBox.Show("Cambios realizados");
          // Refrescar dgAsignaturas
          RefrescarListaAsigs(idAlum);
        }
        catch (Exception ex)
        {
          MessageBox.Show(ex.InnerException.Message);
        }
      }
    }

    private void RefrescarListaAsigs(int idAlum)
    {
      int indice = dgAsignaturas.SelectedIndex;
      CProveedorAsignaturasNotas pAsigsNotas = new CProveedorAsignaturasNotas();
      asigs = pAsigsNotas.ObtenerAsignaturasNotas(idAlum);
      spAsignaturas.DataContext = asigs;
      dgAsignaturas.SelectedIndex = indice;
    }

    private void CambiosBDInsertarAlumno_Click(object sender, RoutedEventArgs e)
    {
      using (bd_notasAlumnosEntities contextoDeObjs =
             new bd_notasAlumnosEntities())
      {
        // Obtener los datos
        int idAlum = 1234560;
        string nomAlum = "Alberto García Sánchez";

        // Crear un nuevo objeto alumno
        alumno alum = alumno.Createalumno(idAlum, nomAlum);

        // Añadirlo al conjunto de entidades alumnos
        contextoDeObjs.AddToalumnos(alum);
        // O bien:
        //contextoDeObjs.AddObject("alumnos", alum);

        // Enviar los cambios a la base de datos
        try
        {
          contextoDeObjs.SaveChanges();
          MessageBox.Show("Cambios realizados");
        }
        catch (Exception ex)
        {
          MessageBox.Show(ex.InnerException.Message);
        }
      }
    }

    private void CambiosBDInsertarAsignatura_Click(object sender, RoutedEventArgs e)
    {
      using (bd_notasAlumnosEntities contextoDeObjs =
             new bd_notasAlumnosEntities())
      {
        // Obtener los datos
        int idAsig = 34680;
        string nomAsig = "PROGRAMACIÓN CONCURRENTE";

        // Crear una nueva asignatura
        asignatura asig = asignatura.Createasignatura(idAsig, nomAsig);

        // Añadirla al conjunto de entidades asignaturas
        contextoDeObjs.AddToasignaturas(asig);

        // Enviar los cambios a la base de datos
        try
        {
          contextoDeObjs.SaveChanges();
          MessageBox.Show("Cambios realizados");
        }
        catch (Exception ex)
        {
          MessageBox.Show(ex.InnerException.Message);
        }
      }
    }

    private void CambiosBDMatricular_Click(object sender, RoutedEventArgs e)
    {
      using (bd_notasAlumnosEntities contextoDeObjs =
             new bd_notasAlumnosEntities())
      {
        // Obtener los datos
        int idAlum = 1234560;
        int idAsig = 34680;

        // Consulta para obtener el alumno a matricular
        var consulta1 = from alum in contextoDeObjs.alumnos
                        where alum.id_alumno == idAlum
                        select alum;

        // Consulta para obtener la asignatura de la que se va a matricular
        var consulta2 = from asig in contextoDeObjs.asignaturas
                        where asig.id_asignatura == idAsig
                        select asig;

        if (consulta1.Count() == 0 || consulta2.Count() == 0)
        {
          MessageBox.Show("El alumno o la asignatura no existen");
          return;
        }
        alumno alumno = consulta1.First();
        asignatura asignatura = consulta2.First();

        // Crear un nuevo objeto alums_asigs
        var al_as = alum_asig.Createalum_asig(idAlum, idAsig, 0.0F);

        // Establecer las relaciones
        al_as.alumnoReference.Value = alumno; // N:1
        al_as.asignaturaReference.Value = asignatura; // N:1
        // O bien:
        //alumno.alums_asigs.Add(al_as);     // 1:N
        //asignatura.alums_asigs.Add(al_as); // 1:N

        // Añadir el nuevo objeto al conjunto de entidades alums_asigs
        contextoDeObjs.AddToalums_asigs(al_as);

        // Enviar los cambios a la base de datos
        try
        {
          contextoDeObjs.SaveChanges();
          MessageBox.Show("Cambios realizados");
        }
        catch (Exception ex)
        {
          MessageBox.Show(ex.InnerException.Message);
        }
      }
    }

    private void CambiosBDBorrarAlumno_Click(object sender, RoutedEventArgs e)
    {
      using (bd_notasAlumnosEntities contextoDeObjs =
             new bd_notasAlumnosEntities())
      {
        // Obtener los datos
        int idAlum = 1234560;

        // Consulta para obtener los objetos secundarios de idAlum
        var consulta1 =
          from alum in contextoDeObjs.alums_asigs
          where alum.id_alumno == idAlum
          select alum;

        if (consulta1.Count() != 0)
        {
          foreach (var al_as in consulta1)
          {
            // Borrar los objetos secundarios
            contextoDeObjs.DeleteObject(al_as);
          }
        }

        // Consulta para obtener el objeto primario idAlum
        var consulta2 =
          from alum in contextoDeObjs.alumnos
          where alum.id_alumno == idAlum
          select alum;

        if (consulta2.Count() != 0)
        {
          foreach (var alum in consulta2)
          {
            // Borrar el objeto primario
            contextoDeObjs.DeleteObject(alum);
          }
        }

        if (consulta1.Count() == 0 && consulta2.Count() == 0)
        {
          MessageBox.Show("La consulta no contiene elementos");
          return;
        }

        // Enviar los cambios a la base de datos
        try
        {
          contextoDeObjs.SaveChanges();
          MessageBox.Show("Cambios realizados");
        }
        catch (Exception ex)
        {
          MessageBox.Show(ex.InnerException.Message);
        }
      }
    }

    private void CambiosBDBorrarAsignatura_Click(object sender, RoutedEventArgs e)
    {
      using (bd_notasAlumnosEntities contextoDeObjs =
             new bd_notasAlumnosEntities())
      {
        // Obtener los datos
        int idAsig = 34680;

        // Consulta para obtener los objetos secundarios de idAsig
        var consulta1 =
          from asig in contextoDeObjs.alums_asigs
          where asig.id_asignatura == idAsig
          select asig;

        if (consulta1.Count() != 0)
        {
          foreach (var al_as in consulta1)
          {
            // Borrar los objetos secundarios
            contextoDeObjs.DeleteObject(al_as);
          }
        }

        // Consulta para obtener el objeto primario idAsig
        var consulta2 =
          from asig in contextoDeObjs.asignaturas
          where asig.id_asignatura == idAsig
          select asig;

        if (consulta2.Count() != 0)
        {
          foreach (var asig in consulta2)
          {
            // Borrar el objeto primario
            contextoDeObjs.DeleteObject(asig);
          }
        }

        if (consulta1.Count() == 0 && consulta2.Count() == 0)
        {
          MessageBox.Show("La consulta no contiene elementos");
          return;
        }

        // Enviar los cambios a la base de datos
        try
        {
          contextoDeObjs.SaveChanges();
          MessageBox.Show("Cambios realizados");
        }
        catch (Exception ex)
        {
          MessageBox.Show(ex.InnerException.Message);
        }
      }
    }

    private void CambiosBDConcurrencia_Click(object sender, RoutedEventArgs e)
    {
      using (bd_notasAlumnosEntities
             contexto1DeObjs = new bd_notasAlumnosEntities(),
             contexto2DeObjs = new bd_notasAlumnosEntities())
      {
        // SESIÓN 1: contexto1DeObjs
        // Obtener los datos
        int idAlum = 1234567;
        int idAsig = 20590;

        // Consulta para obtener la fila a modificar
        var consulta1 =
            from al_as in contexto1DeObjs.alums_asigs
            where al_as.id_alumno == idAlum &&
                  al_as.id_asignatura == idAsig
            select al_as;

        // Ver el código T-SQL
        MessageBox.Show(((ObjectQuery)consulta1).ToTraceString());

        // Ejecutar la consulta y cambiar la nota
        consulta1.First().nota = consulta1.First().nota + 0.5F;

        // SESIÓN 2: contexto2DeObjs
        // Obtener los datos: mismo alumno y misma asignatura
        // pero diferente nota
        idAlum = 1234567;
        idAsig = 20590;

        // Consulta para obtener la fila a modificar
        var consulta2 =
            from al_as in contexto2DeObjs.alums_asigs
            where al_as.id_alumno == idAlum &&
                  al_as.id_asignatura == idAsig
            select al_as;

        // Ejecutar la consulta y cambiar la nota
        consulta2.First().nota = consulta2.First().nota + 0.75F;

        // SESIÓN 1: enviar los cambios a la base de datos
        try
        {
          contexto1DeObjs.SaveChanges();
        }
        catch (Exception ex)
        {
          MessageBox.Show(ex.InnerException.Message);
        }

        // SESIÓN 2: enviar los cambios a la base de datos
        try
        {
          contexto2DeObjs.SaveChanges();
        }
        catch (Exception ex)
        {
          MessageBox.Show(ex.InnerException.Message);
        }
      }
    }

    private void CambiosBDEstado_Click(object sender, RoutedEventArgs e)
    {
      using (bd_notasAlumnosEntities
             contextoDeObjs = new bd_notasAlumnosEntities())
      {
        // Obtener los datos
        int idAlum = 1234567;
        int idAsig = 20590;

        // Consulta para obtener la fila a modificar
        var consulta1 =
            from al_as in contextoDeObjs.alums_asigs
            where al_as.id_alumno == idAlum &&
                  al_as.id_asignatura == idAsig
            select al_as;

        // Ejecutar la consulta y cambiar la nota
        consulta1.First().nota = consulta1.First().nota + 0.5F;

        ObjectStateManager admin = contextoDeObjs.ObjectStateManager;
        var entidadesModificadas =
            admin.GetObjectStateEntries(EntityState.Modified);

        foreach (var OSEntry in entidadesModificadas)
        {
          string info =
            "Nombre de la entidad contenedora: " +
            OSEntry.EntityKey.EntityContainerName + "\n" +
            "Nombre del conjunto de entidades: " +
            OSEntry.EntityKey.EntitySetName.ToString() + "\n" +
            "Clave: " +
            OSEntry.EntityKey.EntityKeyValues.First().Key + "=" +
            OSEntry.EntityKey.EntityKeyValues.First().Value + "\n" +
            "Valores originales -> actuales:\n";

          for (int i = 0; i < OSEntry.OriginalValues.FieldCount; i++)
          {
            info += "\t" + OSEntry.OriginalValues[i] + " -> " +
                    OSEntry.CurrentValues[i] + "\n";
          }
          MessageBox.Show(info);
        }

        try
        {
          contextoDeObjs.SaveChanges();
        }
        catch (Exception ex)
        {
          MessageBox.Show(ex.Message);
        }
      }
    }
  }
}
