﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Threading;

namespace ApSilverlight.Paginas
{
  public partial class SilverlightAudio : UserControl
  {
    private DispatcherTimer temporizador;

    public SilverlightAudio()
    {
      InitializeComponent();
      temporizador = new DispatcherTimer();
      temporizador.Interval = TimeSpan.FromSeconds(0.1);
      temporizador.Tick += Temporizador_Tick;
      sliderPosicion.AddHandler(MouseLeftButtonDownEvent, new MouseButtonEventHandler(sliderPosicion_MouseLeftButtonDown), true);
      sliderPosicion.AddHandler(MouseLeftButtonUpEvent, new MouseButtonEventHandler(sliderPosicion_MouseLeftButtonUp), true);
    }

    void Temporizador_Tick(object sender, EventArgs e)
    {
      tbkEstado.Text = meReproductor.Position.ToString().TrimEnd(new char[] {'0'});
      sliderPosicion.Value = meReproductor.Position.TotalSeconds;
    }

    private void meReproductor_MediaOpened(object sender, RoutedEventArgs e)
    {
      sliderPosicion.Maximum = meReproductor.NaturalDuration.TimeSpan.TotalSeconds;
    }

    private void btReproducir_Click(object sender, RoutedEventArgs e)
    {
      temporizador.Start();
      meReproductor.Play();
    }

    private void btParar_Click(object sender, RoutedEventArgs e)
    {
      meReproductor.Stop();
      temporizador.Stop();
    }

    private void btPausa_Click(object sender, RoutedEventArgs e)
    {
      meReproductor.Pause();
      temporizador.Stop();
    }

    private void meReproductor_MediaEnded(object sender, RoutedEventArgs e)
    {
      if (chbRepetir.IsChecked == true)
      {
        meReproductor.Position = TimeSpan.Zero;
        meReproductor.Play();
      }
      else
        temporizador.Stop();
    }

    private void chbSilencio_Click(object sender, RoutedEventArgs e)
    {
      meReproductor.IsMuted = (bool)chbSilencio.IsChecked;
    }

    private void meReproductor_MediaFailed(object sender, ExceptionRoutedEventArgs e)
    {
      tbkEstado.Text = e.ErrorException.Message;
    }

    private void meReproductor_CurrentStateChanged(object sender, RoutedEventArgs e)
    {
      tbkEstado.Text = meReproductor.CurrentState.ToString();
    }

    private void sliderVolumen_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
    {
      if (meReproductor != null) 
        meReproductor.Volume = sliderVolumen.Value;
    }

    private void sliderBalance_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
    {
      if (meReproductor != null) 
        meReproductor.Balance = sliderBalance.Value;
    }

    private void sliderPosicion_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
    {
      btPausa_Click(btPausa, null);
    }

    private void sliderPosicion_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
    {
      meReproductor.Position = TimeSpan.FromSeconds(sliderPosicion.Value);
      meReproductor.Play();
      temporizador.Start();
    }
  }
}
