﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;
using System.Windows.Data;
using System.Collections.Specialized;
using ColeccionDeDatos;
using System.Windows.Media.Imaging;
using System.Collections.Generic;

namespace SilverlightAccesoColDatos.Views
{
  public partial class PagMostrarPersona : Page
  {
    private PagedCollectionView fuenteDeDatos = null;
    private VenSecAgregarTfno ventanaHija = null;
    
    public PagMostrarPersona()
    {
      InitializeComponent();
      fuenteDeDatos = new PagedCollectionView(
        (Application.Current as App).ColTelefonos);
      LayoutRoot.DataContext = fuenteDeDatos;
    }

    // Se ejecuta cuando el usuario navega a esta página.
    protected override void OnNavigatedTo(NavigationEventArgs e)
    {
      // Manejar el evento CurrentChanged de la vista
      fuenteDeDatos.CurrentChanged +=
        new EventHandler(fuenteDeDatos_CurrentChanged);
      // Manejar el evento CollectionChanged de la vista
      fuenteDeDatos.CollectionChanged +=
        new NotifyCollectionChangedEventHandler(fuenteDeDatos_CollectionChanged);
      // Actualizar posición actual y total elementos
      ActualizarPosicionReg();
    }

    public void fuenteDeDatos_CurrentChanged(object sender, EventArgs e)
    {
      ActualizarPosicionReg();
    }

    private void ActualizarPosicionReg()
    {
      if (fuenteDeDatos.CurrentItem == null) return;
      // Actualizar posición actual y total registros
      string posicionActual = Convert.ToString(fuenteDeDatos.CurrentPosition + 1);
      btxPosicionReg.Text = posicionActual;
      btxTotalReg.Text = fuenteDeDatos.Count.ToString();

      // Poner una foto ficticia si este campo es null
      if ((fuenteDeDatos.CurrentItem as Telefono).Foto != null) return;
      WriteableBitmap wbm = new WriteableBitmap(0, 0).FromResource("Assets/foto.jpg");
      imFoto.Source = wbm;
    }

    private void btPrimero_Click(object sender, RoutedEventArgs e)
    {
      fuenteDeDatos.MoveCurrentToFirst();
    }

    private void btAnterior_Click(object sender, RoutedEventArgs e)
    {
      fuenteDeDatos.MoveCurrentToPrevious();
      if (fuenteDeDatos.IsCurrentBeforeFirst)
        fuenteDeDatos.MoveCurrentToFirst();
    }

    private void btSiguiente_Click(object sender, RoutedEventArgs e)
    {
      fuenteDeDatos.MoveCurrentToNext();
      if (fuenteDeDatos.IsCurrentAfterLast)
        fuenteDeDatos.MoveCurrentToLast();
    }

    private void btUltimo_Click(object sender, RoutedEventArgs e)
    {
      fuenteDeDatos.MoveCurrentToLast();
    }

    private void ctFiltrar_TextChanged(object sender, TextChangedEventArgs e)
    {
      TextBox ct = sender as TextBox;
      if (ct == null) return;
      if (fuenteDeDatos.CanFilter)
      {
        fuenteDeDatos.Filter = delegate(object elemento)
        {
          return ((Telefono)elemento).NumTelefono.Contains(ct.Text);
        };
      }
    }

    public void fuenteDeDatos_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
    {
      ActualizarPosicionReg();
    }

    private void imFoto_MouseLeftButtonDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
    {
      OpenFileDialog dlg = new OpenFileDialog();
      dlg.Filter = "ficheros jpg (*.jpg)|*.jpg|" +
                   "ficheros jpeg (*.jpeg)|*.jpeg|" +
                   "ficheros png (*.png)|*.png|" +
                   "Todos (*.*)|*.*";

      if (dlg.ShowDialog() == true)
      {
        if (dlg.File.Name == null || dlg.File.Name.Length == 0) return;
        WriteableBitmap imagen = new WriteableBitmap(0, 0);
        imagen.SetSource(dlg.File.OpenRead());
        imFoto.Source = imagen;
      }
    }

    private void btAñadir_Click(object sender, RoutedEventArgs e)
    {
      ventanaHija = new VenSecAgregarTfno();
      ventanaHija.Closed += new EventHandler(ventanaHija_Closed);
      ventanaHija.Show();
    }

    private void ventanaHija_Closed(object sender, EventArgs e)
    {
      if (ventanaHija.DialogResult == true)
      {
        if (ventanaHija.NuevoObjTfno != null)
        {
          (fuenteDeDatos.SourceCollection as List<Telefono>).Add(ventanaHija.NuevoObjTfno);
          fuenteDeDatos.MoveCurrentToLast();
          // Añadir el objeto a la base de datos
          // ...
        }
      }
    }

    private void btBorrar_Click(object sender, RoutedEventArgs e)
    {
      // Quitar de la colección el elemento mostrado
      Telefono tfno = (Telefono)fuenteDeDatos.CurrentItem;
      if (tfno != null)
      {
        int siguientePosActual = (fuenteDeDatos.CurrentPosition - 1);
        (fuenteDeDatos.SourceCollection as List<Telefono>).Remove(tfno);
        fuenteDeDatos.Refresh();
        if (siguientePosActual > -1)
          fuenteDeDatos.MoveCurrentToPosition(siguientePosActual);
        // Borrar el objeto de la base de datos
        // ...
      }
    }

    private void btGuardar_Click(object sender, RoutedEventArgs e)
    {
      // Guardar modificaciones en la base de datos
      // ...
    }
  }
}
