﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using ApSilverlightTfnos.ServiceReference1;
using System.ComponentModel;
using System.Windows.Data;
using System.Windows.Media.Imaging;

namespace ApSilverlightTfnos
{
  public partial class MainPage : UserControl
  {
    private Service1Client cliente = null;
    private PagedCollectionView vistaListaDetalles = null;
    private telefono objTelefonoSeleccionado = null;

    public MainPage()
    {
      InitializeComponent();
      cliente = new Service1Client();
      cliente.ObtenerTfnoCompleted += new EventHandler<ObtenerTfnoCompletedEventArgs>(cliente_ObtenerTfnoCompleted);
      cliente.ObtenerTfnosCompleted += new EventHandler<ObtenerTfnosCompletedEventArgs>(cliente_ObtenerTfnosCompleted);
      cliente.ActualizarTfnoCompleted += new EventHandler<AsyncCompletedEventArgs>(cliente_ActualizarTfnoCompleted);
      cliente.AgregarTfnoCompleted += new EventHandler<AsyncCompletedEventArgs>(cliente_AgregarTfnoCompleted);
      cliente.BorrarTfnoCompleted += new EventHandler<AsyncCompletedEventArgs>(cliente_BorrarTfnoCompleted);
    }

    private void UserControl_Loaded(object sender, RoutedEventArgs e)
    {
      cliente.ObtenerTfnosAsync();
    }

    void cliente_ObtenerTfnosCompleted(object sender, ObtenerTfnosCompletedEventArgs e)
    {
      if (e.Error != null)
      {
        tbkInfo.Text = "Error: " + e.Error.Message.ToString();
        popInfo.IsOpen = true;
      }
      else
      {
        vistaListaDetalles = new PagedCollectionView(e.Result);
        vistaListaDetalles.SortDescriptions.Add(new SortDescription(
                        "apellido1", ListSortDirection.Ascending));
        vistaListaDetalles.MoveCurrentToFirst();
        lstPersonas.ItemsSource = vistaListaDetalles;
      }
    }

    private void lstPersonas_SelectionChanged(object sender, SelectionChangedEventArgs e)
    {
      ComboBox lista = sender as ComboBox;
      if (lista.SelectedIndex < 0) return;

      Detalles elementoSeleccionado = lista.SelectedItem as Detalles; // coincide con vistaListaDetalles.CurrentItem
      if (elementoSeleccionado.telefono != null)
        cliente.ObtenerTfnoAsync(elementoSeleccionado.telefono);
    }

    void cliente_ObtenerTfnoCompleted(object sender, ObtenerTfnoCompletedEventArgs e)
    {
      if (e.Error != null)
      {
        tbkInfo.Text = "Error: el teléfono no existe";
        popInfo.IsOpen = true;
      }
      else
      {
        objTelefonoSeleccionado = e.Result;
        // Establecer el contexto de datos del Grid
        LayoutRoot.DataContext = objTelefonoSeleccionado;
        // Poner una foto ficticia si este campo es null
        if (objTelefonoSeleccionado.foto != null) return;
        WriteableBitmap wbm = new WriteableBitmap(0, 0).FromResource("Recursos/foto.jpg");
        imFoto.Source = wbm;
      }
    }

    private void btActualizar_Click(object sender, RoutedEventArgs e)
    {
      cliente.ActualizarTfnoAsync(objTelefonoSeleccionado);
    }

    void cliente_ActualizarTfnoCompleted(object sender, AsyncCompletedEventArgs e)
    {
      if (e.Error != null)
      {
        tbkInfo.Text = "Error: " + e.Error.Message.ToString();
        popInfo.IsOpen = true;
      }
      else
      {
        // Actualizar los datos del ComboBox
        Detalles objDetallesSeleccionado = lstPersonas.SelectedItem as Detalles;
        objDetallesSeleccionado.apellido1 = objTelefonoSeleccionado.apellido1;
        objDetallesSeleccionado.telefono = objTelefonoSeleccionado.telefono1;
        tbkInfo.Text = "Actualizado";
        popInfo.IsOpen = true;
      }
    }

    private void imFoto_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
    {
      OpenFileDialog dlg = new OpenFileDialog();
      dlg.Filter = "ficheros jpg (*.jpg)|*.jpg|" +
                    "ficheros jpeg (*.jpeg)|*.jpeg|" +
                    "ficheros png (*.png)|*.png|" +
                    "Todos (*.*)|*.*";

      if (dlg.ShowDialog() == true)
      {
        if (dlg.File.Name == null || dlg.File.Name.Length == 0) return;
        WriteableBitmap imagen = new WriteableBitmap(0, 0);
        imagen.SetSource(dlg.File.OpenRead());
        imFoto.Source = imagen;
      }
    }

    private void btAgregar_Click(object sender, RoutedEventArgs e)
    {
      cliente.AgregarTfnoAsync(objTelefonoSeleccionado);
    }

    void cliente_AgregarTfnoCompleted(object sender, AsyncCompletedEventArgs e)
    {
      if (e.Error != null)
      {
        tbkInfo.Text = "Error: ¿existe el teléfono?";
        popInfo.IsOpen = true;
      }
      else
      {
        // Añadir un nuevo elemento a la vista
        vistaListaDetalles.AddNew();
        Detalles objDetallesAgregado = vistaListaDetalles.CurrentAddItem as Detalles;
        objDetallesAgregado.apellido1 = objTelefonoSeleccionado.apellido1;
        objDetallesAgregado.telefono = objTelefonoSeleccionado.telefono1;
        vistaListaDetalles.CommitNew();
        tbkInfo.Text = "Añadido";
        popInfo.IsOpen = true;
      }
    }

    private void btBorrar_Click(object sender, RoutedEventArgs e)
    {
      cliente.BorrarTfnoAsync(objTelefonoSeleccionado);
    }

    void cliente_BorrarTfnoCompleted(object sender, AsyncCompletedEventArgs e)
    {
      if (e.Error != null)
      {
        tbkInfo.Text = "Error: ¿existe el teléfono?";
        popInfo.IsOpen = true;
      }
      else
      {
        // Actualizar los datos del ComboBox
        vistaListaDetalles.Remove(vistaListaDetalles.CurrentItem);
        vistaListaDetalles.MoveCurrentToFirst();
        tbkInfo.Text = "Borrado";
        popInfo.IsOpen = true;
      }
    }

    private void popInfo_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
    {
      popInfo.IsOpen = false;
    }
  }
}
