﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using ServicioWcfTfnos;
using System.Data;

/// <summary>
/// Capa BLL (lógica de negocio)
/// </summary>
public class telefonosBll
{
  public telefonosBll() { }

  public telefono ObtenerTfnoBll(string tf)
  {
    using (bd_telefonos3Entities contexto =
           new bd_telefonos3Entities())
    {
      var consulta = from tfno in contexto.telefonos
                     where tfno.telefono1 == tf
                     select tfno;
      telefono registro = consulta.First();
      contexto.Detach(registro);
      return registro;
    }
  }

  public List<Detalles> ObtenerTfnosBll()
  {
    using (bd_telefonos3Entities contexto =
           new bd_telefonos3Entities())
    {
      var consulta = from persona in contexto.telefonos
                     select new
                     {
                       persona.apellido1,
                       persona.telefono1
                     };
      List<Detalles> lista = new List<Detalles>();
      foreach (var e in consulta)
      {
        lista.Add(new Detalles(e.apellido1, e.telefono1));
      }
      return lista;
    }
  }

  public void ActualizarTfnoBll(telefono entidad)
  {
    using (bd_telefonos3Entities contexto =
           new bd_telefonos3Entities())
    {
      // Obtener una entidad conectada con la clave especificada,
      // aplicar los cambios y salvarlos en la base de datos
      contexto.VincularObjeto(entidad); // método extensor 
      contexto.SaveChanges();
    }
  }

  public void AgregarTfnoBll(telefono entidad)
  {
    using (bd_telefonos3Entities contexto =
           new bd_telefonos3Entities())
    {
      // Si no hay una entidad con el mismo teléfono, añadirla
      // al contexto actual y salvar los cambios
      if (ExisteTelefono(entidad, contexto) != null)
        throw new InvalidOperationException("El teléfono existe");
      contexto.telefonos.AddObject(entidad);
      contexto.SaveChanges();
    }
  }

  public void BorrarTfnoBll(telefono entidad)
  {
    using (bd_telefonos3Entities contexto =
           new bd_telefonos3Entities())
    {
      try
      {
        // Obtener la entidad conectada correspondiente, borrarla
        // del contexto actual y salvar los cambios
        if ((entidad = ExisteTelefono(entidad, contexto)) == null)
          throw new InvalidOperationException("El teléfono no existe");
        contexto.DeleteObject(entidad);
        contexto.SaveChanges();
      }
      catch (OptimisticConcurrencyException ex)
      {
        throw new InvalidOperationException(
          "El elemento especificado no puede ser borrado");
      }
    }
  }

  private telefono ExisteTelefono(telefono entidad,
  bd_telefonos3Entities contexto)
  {
    try
    {
      var consulta = from tfno in contexto.telefonos
                     where tfno.telefono1 == entidad.telefono1
                     select tfno;
      telefono registro = consulta.First();
      return registro;
    }
    catch (Exception)
    {
      return null;
    }
  }
}
