﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;

namespace ApNavSilverlight
{
  public partial class MainPage : UserControl
  {
    private Login formLogin = new Login();
    private RefServicioLogin.LoginClient svc = null;

    public MainPage()
    {
      InitializeComponent();
      formLogin.Closed += new EventHandler(formLogin_Closed);

      Uri uri = new Uri(Application.Current.Host.Source,
                          "../Login.svc");
      svc = new RefServicioLogin.LoginClient("CustomBinding_Login", uri.AbsoluteUri);
      svc.AuthenticateCompleted +=
        new EventHandler<RefServicioLogin.AuthenticateCompletedEventArgs>(
          svc_AuthenticateCompleted);
      svc.SignOutCompleted += new EventHandler<System.ComponentModel.AsyncCompletedEventArgs>(svc_SignOutCompleted);
    }

    // Después de la navegación de Frame, asegúrese de que el control HyperlinkButton que representa a la página actual está seleccionado
    private void ContentFrame_Navigated(object sender, NavigationEventArgs e)
    {
      foreach (UIElement child in LinksStackPanel.Children)
      {
        HyperlinkButton hb = child as HyperlinkButton;
        if (hb != null && hb.NavigateUri != null)
        {
          if (hb.NavigateUri.ToString().Equals(e.Uri.ToString()))
          {
            VisualStateManager.GoToState(hb, "ActiveLink", true);
          }
          else
          {
            VisualStateManager.GoToState(hb, "InactiveLink", true);
          }
        }
      }
    }

    // Si se produce un error durante la navegación, mostrar una ventana de error
    private void ContentFrame_NavigationFailed(object sender, NavigationFailedEventArgs e)
    {
      e.Handled = true;
      ChildWindow errorWin = new ErrorWindow(e.Uri);
      errorWin.Show();
    }

    private void hlInicioSesion_Click(object sender, RoutedEventArgs e)
    {
      if (hlInicioSesion.Content.ToString() == "Iniciar sesión")
      {
        formLogin.Show();
      }
      else
      {
        tbBienvenido.Text = "";
        tbSeparador.Visibility = Visibility.Collapsed;
        hlInicioSesion.Content = "Iniciar sesión";
        svc.SignOutAsync();
      }
    }

    private void formLogin_Closed(object sender, EventArgs e)
    {
      if (formLogin.DialogResult == true)
      {
        // Aquí verificaríamos si las credenciales son válidas 
        svc.AuthenticateAsync(formLogin.txtUsuario.Text,
                              formLogin.txtContraseña.Password);
      }
    }

    private void svc_AuthenticateCompleted(object sender, RefServicioLogin.AuthenticateCompletedEventArgs e)
    {
      if (e.Result)
      {
        tbSeparador.Visibility = Visibility.Visible;
        tbBienvenido.Text = "Bienvenido, " + formLogin.txtUsuario.Text;
        hlInicioSesion.Content = "Cerrar sesión";
      }
      else
      {
        ChildWindow errorWin = new ErrorWindow("Error:", "Las credenciales no son correctas");
        errorWin.Show();
      }
    }

    private void svc_SignOutCompleted(object sender, System.ComponentModel.AsyncCompletedEventArgs e)
    {
      if (e.Error != null)
      {
        string s = e.Error.Message;
        ChildWindow errorWin = new ErrorWindow("Error:", s);
        errorWin.Show();
      }
    }
  }
}