﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Navigation;
using System.ComponentModel;
using ApNavSilverlightSvcAps.RefAuthenticationSvc;
using ApNavSilverlightSvcAps.RefRoleSvc;

namespace ApNavSilverlightSvcAps
{
  public partial class MainPage : UserControl
  {
    private Login formLogin = new Login();
    private AuthenticationServiceClient svcAuthentication = null;
    private RoleServiceClient svcRole = null;

    private bool usuarioAutenticado = false;
    private List<string> rolesUsuario = null;

    public MainPage()
    {
      InitializeComponent();
      formLogin.Closed += new EventHandler(formLogin_Closed);
      // Autenticación
      Uri uriSvcAut = new Uri(Application.Current.Host.Source,
                              "../AuthenticationService.svc");
      svcAuthentication = new AuthenticationServiceClient(
        "BasicHttpBinding_AuthenticationService", uriSvcAut.AbsoluteUri);

      svcAuthentication.LoginCompleted +=
        new EventHandler<LoginCompletedEventArgs>(
          svcAuthentication_LoginCompleted);
      svcAuthentication.LogoutCompleted +=
        new EventHandler<AsyncCompletedEventArgs>(
          svcAuthentication_LogoutCompleted);
      svcAuthentication.IsLoggedInCompleted +=
        new EventHandler<IsLoggedInCompletedEventArgs>(
          svcAuthentication_IsLoggedInCompleted);
      // Roles
      Uri uriSvcRol = new Uri(Application.Current.Host.Source,
                              "../RoleService.svc");
      svcRole = new RoleServiceClient(
        "BasicHttpBinding_RoleService", uriSvcRol.AbsoluteUri);
      svcRole.GetRolesForCurrentUserCompleted +=
        new EventHandler<GetRolesForCurrentUserCompletedEventArgs>(
          svcRole_GetRolesForCurrentUserCompleted);
    }

    // Después de la navegación de Frame, asegúrese de que el control HyperlinkButton que representa a la página actual está seleccionado
    private void ContentFrame_Navigated(object sender, NavigationEventArgs e)
    {
      foreach (UIElement child in LinksStackPanel.Children)
      {
        HyperlinkButton hb = child as HyperlinkButton;
        if (hb != null && hb.NavigateUri != null)
        {
          if (hb.NavigateUri.ToString().Equals(e.Uri.ToString()))
          {
            VisualStateManager.GoToState(hb, "ActiveLink", true);
          }
          else
          {
            VisualStateManager.GoToState(hb, "InactiveLink", true);
          }
        }
      }
    }

    // Si se produce un error durante la navegación, mostrar una ventana de error
    private void ContentFrame_NavigationFailed(object sender, NavigationFailedEventArgs e)
    {
      e.Handled = true;
      ChildWindow errorWin = new ErrorWindow(e.Uri);
      errorWin.Show();
    }

    private void hlInicioSesion_Click(object sender, RoutedEventArgs e)
    {
      if (hlInicioSesion.Content.ToString() == "Iniciar sesión")
      {
        formLogin.Show();
      }
      else
      {
        tbBienvenido.Text = "";
        tbSeparador.Visibility = Visibility.Collapsed; ;
        hlInicioSesion.Content = "Iniciar sesión";
        svcAuthentication.LogoutAsync();
      }
    }

    private void formLogin_Closed(object sender, EventArgs e)
    {
      if (formLogin.DialogResult == true)
      {
        // Aquí verificaríamos si las credenciales son válidas 
        svcAuthentication.LoginAsync(formLogin.txtUsuario.Text,
          formLogin.txtContraseña.Password, string.Empty, false);
      }
    }

    private void svcAuthentication_LoginCompleted(object sender, LoginCompletedEventArgs e)
    {
      if (e.Result)
      {
        tbSeparador.Visibility = Visibility.Visible;
        tbBienvenido.Text = "Bienvenido, " + formLogin.txtUsuario.Text;
        hlInicioSesion.Content = "Cerrar sesión";
      }
      else
      {
        ChildWindow errorWin = new ErrorWindow("Error:",
          "Las credenciales no son correctas");
        errorWin.Show();
      }
      svcAuthentication.IsLoggedInAsync();
    }

    private void svcAuthentication_LogoutCompleted(object sender, AsyncCompletedEventArgs e)
    {
      if (e.Error != null)
      {
        string s = e.Error.Message;
        ChildWindow errorWin = new ErrorWindow("Error:", s);
        errorWin.Show();
      }
      svcAuthentication.IsLoggedInAsync();
    }

    private void svcAuthentication_IsLoggedInCompleted(object sender, IsLoggedInCompletedEventArgs e)
    {
      usuarioAutenticado = e.Result;
      if (usuarioAutenticado)
        svcRole.GetRolesForCurrentUserAsync();
    }

    private void svcRole_GetRolesForCurrentUserCompleted(object sender, GetRolesForCurrentUserCompletedEventArgs e)
    {
      rolesUsuario = e.Result.ToList();
    }

    private void ContentFrame_Navigating(object sender, NavigatingCancelEventArgs e)
    {
      // Páginas Home y About
      if (e.Uri.OriginalString == "/Home" ||
          e.Uri.OriginalString == "/About" ||
          e.Uri.OriginalString == "/Users")
        return; // continuar
      // Resto de páginas: Mostrar, Agregar y Borrar
      if (!usuarioAutenticado)
        AccesoDenegado(e);
      // Si está autenticado tiene acceso a Mostrar. Ahora:
      else if (e.Uri.OriginalString != "/Mostrar" && rolesUsuario.Count == 0)
        AccesoDenegado(e);
      else
      {
        // "miembros" o "administradores"
        foreach (string role in rolesUsuario)
        {
          // Ambos pueden Agregar, pero Borrar sólo los administradores
          if (e.Uri.OriginalString == "/Borrar" && role != "administradores")
            AccesoDenegado(e);
        }
      }
    }

    private void AccesoDenegado(NavigatingCancelEventArgs e)
    {
      ErrorWindow window = new ErrorWindow("Acceso denegado",
        "No está autorizado para acceder a esta página");
      window.Show();
      e.Cancel = true;
    }
  }
}